package com.volcengine.apm;

import android.os.Handler;
import android.os.Looper;
import androidx.annotation.NonNull;

import com.apm.insight.Npth;
import com.bytedance.apm.insight.FlutterAgent;

import org.json.JSONException;
import org.json.JSONObject;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** VeApmPlugin */
public class VeApmPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Handler flutterConfigCallbackHandler;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "ve_apm");
    channel.setMethodCallHandler(this);
    flutterConfigCallbackHandler = new Handler(Looper.getMainLooper());
    FlutterAgent.registerConfigListener(new FlutterAgent.IFlutterConfigListener() {
      @Override
      public void onRefresh(final JSONObject config, boolean fromLocal) {
          flutterConfigCallbackHandler.post(new Runnable() {
            @Override
            public void run() {
              if (config != null) {
                channel.invokeMethod("configUpdate", config.toString());
              }
            }
          });
      }

      @Override
      public void onReady() {}
    });
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    try {
      if (call.method.equals("reportToInsight")) {
        String log = call.argument("log");
        FlutterAgent.monitorFlutter(new JSONObject(log));
      } else if (call.method.equals("reportDartError")) {
        String log = call.argument("log");
        Npth.reportDartError(log, null, null, null);
      } else if (call.method.equals("getFlutterConfig")) {
        JSONObject flutterConfig = FlutterAgent.getFlutterConfig();
        if (flutterConfig != null) {
          result.success(flutterConfig.toString());
        } else {
          result.success(null);
        }
      } else {
        result.notImplemented();
      }
    } catch (Exception e) {
      result.error("-1", null, null);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
