package com.example.example;

import android.content.Context;
import android.util.Log;

import com.apm.insight.MonitorCrash;
import com.bytedance.apm.insight.ApmInsight;
import com.bytedance.apm.insight.ApmInsightInitConfig;
import com.bytedance.applog.AppLog;
import com.bytedance.applog.ILogger;
import com.bytedance.applog.InitConfig;
import com.bytedance.applog.util.UriConstants;
import com.example.utils.SpUtil;


public class MyApp extends io.flutter.app.FlutterApplication {

    private static final String TAG = "MyApp";

    private InitConfig config;

    // 火山引擎注册APP生成的appId，appid如不清楚请联系客户成功经理
    private String appId = "187277";
    // 用户的user_id，可以是用户登录账号id，也可以是业务方自己定义的user_id
    // 主要用途是将平台数据和用户关联起来，便于排查问题
    private String userId = "98765";
    // app渠道信息，主要用来区分此app是从哪个渠道下载
    private String channel = "google_play";
    //配置数据上报的域名 （私有化部署才需要配置，内部有默认域名）
    private String reportUrl = null;

    private static Context context;
    public static Context getContext() {
        return context;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        context = this;
        appId = SpUtil.getAppId(this);
        reportUrl = SpUtil.getReportUrl(this);
        initConfig();
        initAppLog();
        initCrash();
        initApmInsight();
    }

    /**
     * 配置信息初始化
     */
    private void initConfig() {
        config = new InitConfig(appId, channel);
        // 上报域名只支持中国
        config.setUriConfig(UriConstants.DEFAULT);
        // 是否在控制台输出日志，可用于观察用户行为日志上报情况
        config.setLogger(new ILogger() {
            @Override
            public void log(String msg, Throwable t) {
                Log.d(TAG, msg, t);
            }
        });
        config.setAutoStart(true);
    }

    /**
     * AppLog初始化
     */
    private void initAppLog() {
        AppLog.init(this, config);
    }

    /**
     * 崩溃监控初始化
     */
    private void initCrash() {
        MonitorCrash crash = MonitorCrash.init(this, appId, BuildConfig.VERSION_CODE,
                BuildConfig.VERSION_NAME);
        if (reportUrl != null && !reportUrl.equals("默认值")) {
            Log.i(TAG, "私有化部署地址为：" + reportUrl);
            crash.setReportUrl(reportUrl); // 私有化部署专用, 否则不要动
        }
        crash.config().setChannel(channel).setDeviceId(AppLog.getDid()).setUID(userId);
    }

    /**
     * ApmInsight初始化
     */
    private void initApmInsight() {
        ApmInsightInitConfig.Builder builder = ApmInsightInitConfig.builder();
        // 设置分配的appid
        builder.aid(appId);
        // 支持用户的user_id把平台数据和用户关联起来
        builder.userId(userId);
        // 是否打印日志，注：线上release版本要配置为false
        builder.debugMode(true);
        //配置数据上报的域名 （私有化部署才需要配置，内部有默认域名）
        if (reportUrl != null && !reportUrl.equals("默认值")) {
            Log.i(TAG, "私有化部署地址为：" + reportUrl);
            builder.defaultReportDomain(reportUrl);
        }
        // 初始化ApmInsight
        ApmInsight.getInstance().init(this, builder.build());
    }
}
