#import "VeApmPlugin.h"

#import <Foundation/Foundation.h>
#import "RangersAPM+Flutter.h"

NSString* flutterConfig;

@implementation VeApmPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"ve_apm"
                                                                binaryMessenger:[registrar messenger]];
    VeApmPlugin* instance = [[VeApmPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    // Register callback for recieving config information
    [RangersAPM setDartConfigUpdateHandler:^(NSDictionary * _Nullable config) {
        if (nil != config) {
            NSError *error; 
            NSData *jsonData = [NSJSONSerialization dataWithJSONObject:config 
                                                    options:0 // Pass 0 if you don't care about the readability of the generated string
                                                    error:&error];
            if (error) {
                @throw [NSException exceptionWithName:[@(error.code) stringValue] reason:error.domain userInfo:error.userInfo];
            }
            flutterConfig = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
            [channel invokeMethod: @"configUpdate" arguments: flutterConfig];
        }
     }];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        @try {
            if([@"reportDartError" isEqualToString:call.method]) {
                NSDictionary *arguments = [call arguments];
                NSString *log = arguments[@"log"];
                [RangersAPM recordDartErrorWithTraceStack:log customData:@{} customLog:@"" filters:@{}];
            } else if ([@"reportToInsight" isEqualToString:call.method]) {
                NSDictionary *arguments = [call arguments];
                NSString *log = arguments[@"log"];
                [self reportToInsight:log];
            } else if ([@"getFlutterConfig" isEqualToString:call.method]) {
                result(flutterConfig);
            } else {
                result(FlutterMethodNotImplemented);
            }
        } @catch (NSException *e) {
            result([FlutterError errorWithCode:e.name message:e.reason details:e.userInfo]);
        }
    });
}

- (void)reportToInsight:(NSString*)json {
    NSError *error;
    NSData *data = [json dataUsingEncoding:NSUTF8StringEncoding];
    NSDictionary *log = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:&error];
    if (error) {
        @throw [NSException exceptionWithName:[@(error.code) stringValue] reason:error.domain userInfo:error.userInfo];
    }
    [RangersAPM recordPerformanceLog: log];
}

@end
