import 'dart:async';
import 'dart:io';

import 'package:ve_apm/src/report/apm_reporter.dart';
import 'package:ve_apm/src/plugin/apm_insight.dart';
import 'package:ve_apm/src/utils/log.dart';
import 'package:ve_apm/ve_apm.dart';
import 'package:ve_apm/src/perf/fps_monitor.dart';
import 'package:ve_apm/src/perf/page_load_data_manager.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ApmWidgetsBindingObserver with WidgetsBindingObserver {
  ApmWidgetsBindingObserver({@visibleForTesting this.mockApmReporter,
    @visibleForTesting this.mockPageLoadDataManager,
    @visibleForTesting this.mockFpsMonitor,
    @visibleForTesting this.mockIsAndroid,
    @visibleForTesting this.mockIsIOS});

  Timer? _timer;
  bool _needSessionReport = false;

  @visibleForTesting
  final ApmReporter? mockApmReporter;
  @visibleForTesting
  final PageLoadDataManager? mockPageLoadDataManager;
  @visibleForTesting
  final FpsMonitor? mockFpsMonitor;
  @visibleForTesting
  final bool? mockIsAndroid;
  @visibleForTesting
  final bool? mockIsIOS;

  @visibleForTesting
  Timer? get timer => _timer;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    switch (state) {
      case AppLifecycleState.resumed:
        if (_needSessionReport) {
          final ApmReporter fpmReporter = mockApmReporter ?? ApmReporter.instance;
          logRelease('method: ApmWidgetsBindingObserver.didChangeAppLifecycleState, step: FpmReporter.reportEvent, param: ${Switcher.SESSION.value}');
          fpmReporter.reportEvent(Switcher.SESSION.value);
        }
        _needSessionReport = false;
        _timer?.cancel();
        _timer = null;
        final FpsMonitor fpsMonitor = mockFpsMonitor ?? FpsMonitor.instance;
        fpsMonitor.onAppLifecycleResumed();
        break;
      case AppLifecycleState.paused:
        if (mockIsAndroid ?? Platform.isAndroid) {
          _timer = Timer(const Duration(seconds: 30),() {
            _needSessionReport = true;
          });
        } else if (mockIsIOS ?? Platform.isIOS) {
          _needSessionReport = true;
        }
        final PageLoadDataManager pageLoadDataManager = mockPageLoadDataManager ?? PageLoadDataManager.instance;
        pageLoadDataManager.reportAllPageLoadData();
        final FpsMonitor fpsMonitor = mockFpsMonitor ?? FpsMonitor.instance;
        fpsMonitor.onAppLifecyclePaused();
        break;
      case AppLifecycleState.detached:
        _needSessionReport = false;
        _timer?.cancel();
        _timer = null;
        break;
      default:
        break;
    }
  }
}