import 'package:ve_apm/src/core/constants.dart';
import 'package:ve_apm/src/core/apm_log_report.dart';
import 'package:ve_apm/src/core/apm_navigator_observer.dart';
import 'package:ve_apm/src/errortrace/apm_error_widget.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:ve_apm/src/utils/log.dart' hide kDebugMode;

class ApmErrorTrace {
  static ApmErrorTrace? _instance;

  static ApmErrorTrace get instance => _instance ??= ApmErrorTrace._internal();

  ApmErrorTrace._internal();

  bool isBlank = false;

  Future<void> recordError(FlutterErrorDetails error) async {
    if (kDebugMode) {
      FlutterError.dumpErrorToConsole(error, forceReport: true);
    }

    final Iterable<DiagnosticsNode> info =
        error.informationCollector == null ? [] : error.informationCollector!();
    final StackTrace _stackTrace = error.stack ?? StackTrace.current;
    final String _reason = error.context.toString();
    final String _information =
        info.isEmpty ? '' : (StringBuffer()..writeAll(info, '\n')).toString();

    logRelease('method: ApmErrorTrace.recordError, step: ApmErrorTrace.reprotError, param: ${error.exceptionAsString()};$_reason;$_information;${_stackTrace.toString()}');
    return reportError(
      exception: error.exceptionAsString(),
      reason: _reason,
      information: _information,
      stackTrace: _stackTrace.toString(),
    );
  }

  Map<String, dynamic> generateBaseErrorInfo(
      {String? exception,
        String? reason,
        String? information,
        String? stackTrace}) {
    String? currentRoute = ApmNavigatorObserver.instance.getCurrentRouteName();
    return <String, dynamic>{
      KEY_CRASH_TIME: new DateTime.now().millisecondsSinceEpoch,
      KEY_EXCEPTION: exception ?? '',
      KEY_REASON: reason ?? '',
      KEY_INFORMATION: information ?? '',
      KEY_STACKTRACE: stackTrace ?? '',
      KEY_ROUTE: currentRoute,
      KEY_RECENT_ROUTE:
      ApmNavigatorObserver.instance.getRecentRouteHistory().join("\n")
    };
  }

  Future<void> reportError(
      {String? exception,
      String? reason,
      String? information,
      String? stackTrace}) async {
    if (kDebugMode) {
      print('reportError, exception $exception, stacktrace $stackTrace');
    }

    Map<String, dynamic> logData = generateBaseErrorInfo(
        exception: exception,
        reason: reason,
        information: information,
        stackTrace: stackTrace);

    await Future.delayed(const Duration(milliseconds: 1), () {});

    logData.addAll(<String, dynamic>{
      KEY_FILTERS: <String, dynamic>{KEY_BLANK: isBlank ? '1' : '0'}
    });
    isBlank = false;
    logRelease('method: ApmErrorTrace.reportError, step: LogSend.reportLog, param: $logData');
    LogSend.reportLog(logData, true);
  }

  void updateWidgetErrorInfoFunc() {
    isBlank = true;
  }

  /// Flutter build 异常时自定义ErrorWidget
  void replaceErrorWidget() {
    ErrorWidget.builder = (FlutterErrorDetails details) {
      return defaultApmErrorWidgetBuilder(details);
    };
  }
}
