@Timeout(Duration(minutes: 3))
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:ve_apm/src/core/channel_info_manager.dart';
import 'package:ve_apm/src/plugin/apm_insight.dart';
import 'package:ve_apm/src/report/apm_reporter.dart';

import 'channel_info_manager_test.mocks.dart';

@GenerateMocks([ApmReporter])
void main() {
  group('ChannelInfoManger class test', () {
    tearDownAll(() {
      ChannelInfoManager.instance.clearData();
      ChannelInfoManager.instance.mockApmReporter = null;
    });

    test('enqueue: 两分钟，会定时上报；上报完timer置空', () async {
      ChannelInfo info = ChannelInfo(
          channelName: 'test',
          interceptType: InterceptType.SEND,
          startTimestamp: 0);
      info.identifyType = IdentityType.USER;
      info.timeCost = 1;
      final mockApmReporter = MockApmReporter();
      ChannelInfoManager.instance.mockApmReporter = mockApmReporter;

      ChannelInfoManager.instance.enqueue(info);
      expect(ChannelInfoManager.instance.reportTimer != null, true);
      expect(ChannelInfoManager.instance.dataSize, 1);

      await Future<void>.delayed(const Duration(minutes: 2));
      verify(mockApmReporter.reportEvent(Switcher.CHANNEL.value,
              metric: anyNamed('metric')))
          .called(1);
      expect(ChannelInfoManager.instance.dataSize, 0);
      expect(ChannelInfoManager.instance.reportTimer == null, true);
    });

  });
}
