@Timeout(Duration(seconds: 100))

import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:ve_apm/src/core/apm_widgets_binding_observer.dart';
import 'package:ve_apm/src/report/apm_reporter.dart';
import 'package:ve_apm/src/perf/fps_monitor.dart';
import 'package:ve_apm/src/perf/page_load_data_manager.dart';
import 'package:ve_apm/src/plugin/apm_insight.dart';

import 'widgets_binding_observer_test.mocks.dart';

@GenerateMocks([ApmReporter, PageLoadDataManager, FpsMonitor])
void main() {
  group('ApmWidgetsBindingObserver class test', () {
    test('android: 切换到后台30s以上，返回前端会才会上报session一次', () async{
      final mockApmReporter = MockApmReporter();
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(
          mockApmReporter: mockApmReporter, mockIsAndroid: true);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      await Future<void>.delayed(const Duration(seconds: 29));
      observer.didChangeAppLifecycleState(AppLifecycleState.resumed);
      verifyNever(mockApmReporter.reportEvent(Switcher.SESSION.value));

      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      await Future<void>.delayed(const Duration(seconds: 30));
      observer.didChangeAppLifecycleState(AppLifecycleState.resumed);
      verify(mockApmReporter.reportEvent(Switcher.SESSION.value)).called(1);
    });
    test('android: 切换到后台返回前端后，timer计时器停止计时', () {
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(mockIsAndroid: true);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      expect(observer.timer!.isActive, true);

      observer.didChangeAppLifecycleState(AppLifecycleState.resumed);
      expect(observer.timer == null, true);
    });
    test('android: 生命周期detached后，timer置空', () {
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(mockIsAndroid: true);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      expect(observer.timer!.isActive, true);

      observer.didChangeAppLifecycleState(AppLifecycleState.detached);
      expect(observer.timer == null, true);
    });

    test('iOS: 切换到后台后，返回前端会上报session一次', () {
      final mockApmReporter = MockApmReporter();
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(
          mockApmReporter: mockApmReporter, mockIsIOS: true);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      observer.didChangeAppLifecycleState(AppLifecycleState.resumed);
      verify(mockApmReporter.reportEvent(Switcher.SESSION.value)).called(1);
    });

    test('paused 状态时, 会调用FpsMonitor#onAppLifecyclePaused', () {
      final mockFpsMonitor = MockFpsMonitor();
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(
          mockFpsMonitor: mockFpsMonitor);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      verify(mockFpsMonitor.onAppLifecyclePaused()).called(1);
    });

    test('resumed 状态时, 会调用FpsMonitor#onAppLifecycleResumed', () {
      final mockFpsMonitor = MockFpsMonitor();
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(
          mockFpsMonitor: mockFpsMonitor);
      observer.didChangeAppLifecycleState(AppLifecycleState.resumed);
      verify(mockFpsMonitor.onAppLifecycleResumed()).called(1);
    });

    test('App paused 状态时, 会调用PageLoadDataManager#reportAllPageLoadData', () {
      final mockPageLoadDataManager = MockPageLoadDataManager();
      ApmWidgetsBindingObserver observer = ApmWidgetsBindingObserver(
          mockPageLoadDataManager: mockPageLoadDataManager);
      observer.didChangeAppLifecycleState(AppLifecycleState.paused);
      verify(mockPageLoadDataManager.reportAllPageLoadData()).called(1);
    });

  });
}