import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:ve_apm/src/network/apm_network_data_manager.dart';
import 'package:ve_apm/src/network/apm_network_error.dart';
import 'package:ve_apm/src/plugin/apm_insight.dart';
import 'package:ve_apm/src/report/apm_reporter.dart';

import 'apm_network_data_manager_test.mocks.dart';

class MockSwitcher extends Mock{}

@GenerateMocks([ApmReporter])
void main() {

  group('NetworkDataReporter class test', () {

    test('reportNetworkEvent: 如果是成功的请求并且没命中采样，那么不进行上报，方法提前返回', () {
      NetworkDataItem dataItem = NetworkDataItem.empty();
      dynamic mockSwitcher = MockSwitcher();
      final mockApmReporter = MockApmReporter();
      when(mockSwitcher.isModuleReportable(Switcher.NETWORK) as bool?).thenReturn(false);
      NetworkDataReporter.instance.reportNetworkEvent(NetworkLogTye.SUCCESS_REQUEST, dataItem,
          mockSwitcher: mockSwitcher, mockApmReporter: mockApmReporter);
      verifyNever(mockApmReporter.reportEvent(Switcher.NETWORK.value, metric: anyNamed('metric')));
    });

    test('reportNetworkEvent: 如果是异常请求，无论采样是否命中，都需要上报', () {
      NetworkDataItem dataItem = NetworkDataItem.empty();
      dataItem.uri = 'http://aaa.com';
      dataItem.method = 'test';
      dataItem.callStackTrace = '...';
      dynamic mockSwitcher = MockSwitcher();
      final mockApmReporter = MockApmReporter();
      when(mockSwitcher.isModuleReportable(Switcher.NETWORK) as bool?).thenReturn(false);
      NetworkDataReporter.instance.reportNetworkEvent(NetworkLogTye.STATUS_CODE_ERROR, dataItem,
          mockSwitcher: mockSwitcher, mockApmReporter: mockApmReporter);
      verify(mockApmReporter.reportEvent(Switcher.NETWORK.value, metric: anyNamed('metric'))).called(1);

      NetworkDataReporter.instance.reportNetworkEvent(NetworkLogTye.REQUEST_ERROR, dataItem,
          mockSwitcher: mockSwitcher, mockApmReporter: mockApmReporter);
      verify(mockApmReporter.reportEvent(Switcher.NETWORK.value, metric: anyNamed('metric'))).called(1);
    });
  });
}

