@Timeout(Duration(seconds: 60))
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:ve_apm/src/perf/page_load_data_manager.dart';
import 'package:ve_apm/src/plugin/apm_insight.dart';
import 'package:ve_apm/src/report/apm_reporter.dart';

import 'page_load_data_manager_test.mocks.dart';

@GenerateMocks([ApmReporter])
void main() {
  group('PageLoadDataManager class test', () {
    tearDownAll(() {
      PageLoadDataManager.instance.pageLoadDataCache.clear();
      PageLoadDataManager.instance.mockApmReporter = null;
    });

    test('joinPageLoadData: 相同pageRouteHashCode数据会覆盖', () async {
      PageLoadDataManager.instance
          .joinPageLoadData(1, 'test_page0', 1);
      PageLoadDataManager.instance
          .joinPageLoadData(1, 'test_page1', 2);
      expect(PageLoadDataManager.instance.pageLoadDataCache.length, 1);
      PageLoadData pageLoadData = PageLoadDataManager.instance.pageLoadDataCache[1]!;
      expect(pageLoadData.page, 'test_page1');
      expect(pageLoadData.pageDidPushTime, 2);

      PageLoadDataManager.instance
          .joinPageLoadData(2, 'test_page2', 2);
      expect(PageLoadDataManager.instance.pageLoadDataCache.length, 2);
    });

    test('timer report，并且最后一个数据不会被上报', () async {
      String page = 'test_page';
      num pageDidPushTime = 1;
      final mockApmReporter = MockApmReporter();
      PageLoadDataManager.instance.mockApmReporter = mockApmReporter;
      PageLoadDataManager.instance.joinPageLoadData(1, page, pageDidPushTime);
      PageLoadDataManager.instance.joinPageLoadData(2, page, pageDidPushTime);
      expect(PageLoadDataManager.instance.pageLoadDataCache.length, 2);

      expect(PageLoadDataManager.instance.reportTimer != null, true);
      await Future<void>.delayed(const Duration(seconds: 30));
      verify(mockApmReporter.reportEvent(Switcher.PAGE_LOAD.value,
              timestamp: anyNamed('timestamp'),
              category: anyNamed('category'),
              metric: anyNamed('metric'),
              extra: anyNamed('extra')))
          .called(1);
      expect(PageLoadDataManager.instance.pageLoadDataCache.length, 1);
    });

    test('reportAllPageLoadData 会上报所有数据', () async{
      String page = 'test_page';
      num pageDidPushTime = 1;
      final mockApmReporter = MockApmReporter();
      PageLoadDataManager.instance.mockApmReporter = mockApmReporter;
      PageLoadDataManager.instance.joinPageLoadData(1, page, pageDidPushTime);
      PageLoadDataManager.instance.joinPageLoadData(2, page, pageDidPushTime);
      int length = PageLoadDataManager.instance.pageLoadDataCache.length;
      expect(length, 2);

      PageLoadDataManager.instance.reportAllPageLoadData();
      verify(mockApmReporter.reportEvent(Switcher.PAGE_LOAD.value,
      timestamp: anyNamed('timestamp'),
      category: anyNamed('category'),
      metric: anyNamed('metric'),
      extra: anyNamed('extra')))
          .called(length);
      expect(PageLoadDataManager.instance.pageLoadDataCache.length, 0);
      expect(PageLoadDataManager.instance.reportTimer, null);
    });
  });
}
