import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:mockito/annotations.dart';

import 'package:ve_apm/src/plugin/apm_insight.dart';
import 'apm_insight_test.mocks.dart';

@GenerateMocks([MethodChannel,])
void main() {

  group('Switcher class test', (){
    test('verify Switcher instances\' value', () {
      expect(Switcher.CRASH.value, 'flutter_crash');
      expect(Switcher.SESSION.value, 'flutter_session');
      expect(Switcher.PAGE_LOAD.value, 'flutter_page_load');
      expect(Switcher.FMP.value, 'flutter_fmp');
      expect(Switcher.FPS.value, 'flutter_fps');
      expect(Switcher.NETWORK.value, 'flutter_network');
    });


    test('before method updateReportableModules is executed, isModuleReportable is always false', (){
      expect(Switcher.isModuleReportable(Switcher.CRASH), false);
      expect(Switcher.isModuleReportable(Switcher.SESSION), false);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), false);
      expect(Switcher.isModuleReportable(Switcher.FMP), false);
      expect(Switcher.isModuleReportable(Switcher.FPS), false);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), false);
    });


    test('before method updateReportableModules is executed, fmpSwitch is always true', (){
      expect(Switcher.fmpSwitch, true);
    });

    test('when switcher == 0, module switcher is false', () {
      Map config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 0,
          "flutter_fps_ratio": 1
        },
        "flutter_crash_module":{
          "switcher": 0,
        },
        "flutter_page_load_module":{
          "switcher": 0,
          "flutter_page_load_ratio": 1,
          "flutter_fmp_ratio": 1,
        },
        "flutter_network_module": {
          "switcher": 0,
          "flutter_network_ratio": 1,
        }
      };
      Switcher.updateReportableModules(config);
      expect(Switcher.isModuleReportable(Switcher.CRASH), false);
      expect(Switcher.isModuleReportable(Switcher.SESSION), false);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), false);
      expect(Switcher.isModuleReportable(Switcher.FMP), false);
      expect(Switcher.isModuleReportable(Switcher.FPS), false);
      expect(Switcher.fmpSwitch, false);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), false);
      Switcher.clear();

      config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 0,
          "flutter_fps_ratio": 0
        },
        "flutter_crash_module":{
          "switcher": 0,
        },
        "flutter_page_load_module":{
          "switcher": 0,
          "flutter_page_load_ratio": 0,
          "flutter_fmp_ratio": 0,
        },
      };
      Switcher.updateReportableModules(config);
      expect(Switcher.isModuleReportable(Switcher.CRASH), false);
      expect(Switcher.isModuleReportable(Switcher.SESSION), false);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), false);
      expect(Switcher.isModuleReportable(Switcher.FMP), false);
      expect(Switcher.isModuleReportable(Switcher.FPS), false);
      expect(Switcher.fmpSwitch, false);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), false);
      Switcher.clear();
    });

    test('when switcher == 1 && **_ratio == 0, module switcher is false', () {
      Map config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 1,
          "flutter_fps_ratio": 0
        },
        "flutter_crash_module":{
          "switcher": 1,
          "flutter_crash_ratio": 0,
        },
        "flutter_page_load_module":{
          "switcher": 1,
          "flutter_page_load_ratio": 0,
          "flutter_fmp_ratio": 0,
        },
        "flutter_network_module": {
          "switcher": 1,
          "flutter_network_ratio": 0,
        }
      };
      Switcher.updateReportableModules(config);
      expect(Switcher.isModuleReportable(Switcher.CRASH), false);
      expect(Switcher.isModuleReportable(Switcher.SESSION), false);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), false);
      expect(Switcher.isModuleReportable(Switcher.FMP), false);
      expect(Switcher.isModuleReportable(Switcher.FPS), false);
      expect(Switcher.fmpSwitch, false);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), false);
      Switcher.clear();
    });

    test('when switcher == 1 && without **_ratio, module switcher is true', () {
      Map config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 1,
        },
        "flutter_crash_module":{
          "switcher": 1,
        },
        "flutter_page_load_module":{
          "switcher": 1,
        },
        "flutter_network_module": {
          "switcher": 1,
        }
      };
      Switcher.updateReportableModules(config);
      expect(Switcher.isModuleReportable(Switcher.CRASH), true);
      expect(Switcher.isModuleReportable(Switcher.SESSION), true);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), true);
      expect(Switcher.isModuleReportable(Switcher.FMP), true);
      expect(Switcher.isModuleReportable(Switcher.FPS), true);
      expect(Switcher.fmpSwitch, true);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), true);
      Switcher.clear();
    });

    test('when switcher == 1 && **_ratio == 1, module switcher is true', () {
      Map config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 1,
          "flutter_fps_ratio": 1
        },
        "flutter_crash_module":{
          "switcher": 1,
        },
        "flutter_page_load_module":{
          "switcher": 1,
          "flutter_page_load_ratio": 1,
          "flutter_fmp_ratio": 1,
        },
        "flutter_network_module": {
          "switcher": 1,
          "flutter_network_ratio": 1,
        }
      };
      Switcher.updateReportableModules(config);
      expect(Switcher.isModuleReportable(Switcher.CRASH), true);
      expect(Switcher.isModuleReportable(Switcher.SESSION), true);
      expect(Switcher.isModuleReportable(Switcher.PAGE_LOAD), true);
      expect(Switcher.isModuleReportable(Switcher.FMP), true);
      expect(Switcher.isModuleReportable(Switcher.FPS), true);
      expect(Switcher.fmpSwitch, true);
      expect(Switcher.isModuleReportable(Switcher.NETWORK), true);
      Switcher.clear();
    });

  });
  
  group('ApmInsight class test', () {

    test('before call other functions in class, must call \'ApmInsight.init()\' first', () {
      final Map<String,dynamic> eventMap = <String,dynamic>{};
      Matcher matcher = predicate((e) => e is AssertionError && e.message == 'must call ApmInsight.init() first');
      expect(() => ApmInsight.reportToInsight(eventMap), throwsA(matcher));
      expect(() => ApmInsight.reportDartError(eventMap), throwsA(matcher));

      WidgetsFlutterBinding.ensureInitialized();
      final mockChannel = MockMethodChannel();
      when(mockChannel.invokeMethod<String>(any, any)).thenAnswer((_) => Future.value(null));
      ApmInsight.init(mockChannel: mockChannel);
      expect(() => ApmInsight.reportToInsight(eventMap), returnsNormally);
      expect(() => ApmInsight.reportDartError(eventMap), returnsNormally);
    });

    test('before method configUpdate is executed, report log will be cached', () {
      Map<String, dynamic> eventMap = <String,dynamic>{
        "service": Switcher.PAGE_LOAD.value,
        "timestamp": DateTime.now().millisecondsSinceEpoch,
        "extra": {
          "a": 123
        }
      };
      final mockChannel = MockMethodChannel();
      ApmInsight.reportToInsight(eventMap, mockChannel: mockChannel);
      ApmInsight.reportDartError(eventMap, mockChannel: mockChannel);
      verifyNever(mockChannel.invokeMethod<dynamic>(any, any));
      expect(ApmInsight.crashCache.isNotEmpty, true);
      expect(ApmInsight.eventCache.isNotEmpty, true);
    });

    test('after method configUpdate is executed, cached log will be reported', () {
      Map config = <String,dynamic>{
        "flutter_fps_module": {
          "switcher": 1,
          "flutter_fps_ratio": 1
        },
        "flutter_crash_module":{
          "switcher": 1,
        },
        "flutter_page_load_module":{
          "switcher": 1,
          "flutter_page_load_ratio": 1,
          "flutter_fmp_ratio": 1,
        },
        "flutter_network_module": {
          "switcher": 1,
          "flutter_network_ratio": 1,
        }
      };
      String configStr = json.encode(config);
      MethodCall methodCall = MethodCall("configUpdate", configStr);
      final mockChannel = MockMethodChannel();
      when(mockChannel.invokeMethod<dynamic>(any, any)).thenAnswer((_) => Future<dynamic>.value());
      int len = ApmInsight.crashCache.length + ApmInsight.eventCache.length;
      ApmInsight.handleMethodCall(methodCall, mockChannel: mockChannel);
      expect(ApmInsight.crashCache.isEmpty, true);
      expect(ApmInsight.eventCache.isEmpty, true);
      verify(mockChannel.invokeMethod<dynamic>(any, any)).called(len);
    });
    
    test('If the module does not support reporting, the data of the module will be discarded', () {
      Switcher.clear();
      Map<String,dynamic> eventMap = <String,dynamic>{
        "service": Switcher.PAGE_LOAD.value,
        "timestamp": DateTime.now().millisecondsSinceEpoch,
        "extra": {
          "a": 123
        }
      };
      final mockChannel = MockMethodChannel();
      ApmInsight.reportToInsight(eventMap, mockChannel: mockChannel);
      ApmInsight.reportDartError(eventMap, mockChannel: mockChannel);

      Map config = <String,dynamic>{
        "flutter_crash_module":{
          "switcher": 0,
        },
        "flutter_page_load_module":{
          "switcher": 0,
        },
      };
      String configStr = json.encode(config);
      MethodCall methodCall = MethodCall("configUpdate", configStr);
      ApmInsight.handleMethodCall(methodCall, mockChannel: mockChannel);
      // 数据被清空了，但是上报次数为0
      expect(ApmInsight.crashCache.isEmpty, true);
      expect(ApmInsight.eventCache.isEmpty, true);
      verifyNever(mockChannel.invokeMethod<dynamic>(any, any));
    });

  });
}